/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "TestExecWriteRequest.hpp"

#include "Statement.hpp"

#include "CarobException.hpp"
#include "Common.hpp"

#include <string>
#include <pthread.h>

using std::wstring;

using namespace CarobNS;

void TestExecWriteRequest::testWriteBadRequest()
{
  const wstring fctName(L"TestExecWriteRequest::testWriteBadRequest");
  Statement* statementPtr = NULL;
  try
  {
    statementPtr = connectionPtr->createStatement();
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing dummy update - should fail");
    }
    statementPtr->executeUpdate(L"dummy request");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (ControllerException ce)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Update failed (this is ok). Exception: "
          + ce.description());
    }
  }
  connectionPtr->deleteStatement(statementPtr);
}

void TestExecWriteRequest::testWriteBadTable()
{
  const wstring fctName(L"TestExecWriteRequest::testWriteBadTable");
  Statement* statementPtr = NULL;
  try
  {
    statementPtr = connectionPtr->createStatement();
    if (isInfoEnabled())
    {
      logInfo(fctName, L"Executing bad table update - this should fail");
    }
    statementPtr->executeUpdate(L"UPDATE dummy SET name='gotit' WHERE id=0");
    // We should receive an exception instead of coming here
    CPPUNIT_ASSERT(false);
  }
  catch (BackendException be)
  {
    if (isErrorEnabled())
    {
      logError(fctName, L"Update failed (this is ok). Exception: "
          + be.description());
    }
  }
  connectionPtr->deleteStatement(statementPtr);
}

void TestExecWriteRequest::testWriteGood()
{
  const wstring fctName(L"TestExecWriteRequest::testWriteGood");
  Statement* statementPtr = connectionPtr->createStatement();
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing update - should succeed");
  }
  int nbRowsAffected = statementPtr->executeUpdate(
      L"UPDATE product SET name='changed by testExecuteUpdateGood' WHERE id=0");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Update succeeded. Number of affected rows = "
        + toUserString(nbRowsAffected));
  }
  CPPUNIT_ASSERT(nbRowsAffected == 1);
  connectionPtr->deleteStatement(statementPtr);
}

#define NB_THREADS 50
#define NB_WRITES 10
#define THREAD_TABLE L"thread_table"
#define THREAD_TABLE_TYPE L"(i integer, s varchar(200))"

struct threadArg
{
  TestExecWriteRequest* instancePtr;
  int threadId;
};

void *doThreadedWrites(void *arg)
{
  threadArg* me = (threadArg*)arg;
  const wstring fctName(L"TestExecWriteRequest::testThreadedWrite thread#"
      + toWString(me->threadId));

  Statement* statementPtr = me->instancePtr->connectionPtr->createStatement();
  for (int i = 0; i < NB_WRITES; i++)
  {
    int nbRowsAffected = statementPtr->executeUpdate(
      wstring(L"INSERT INTO ") + THREAD_TABLE + L" VALUES ("
          + toWString(me->threadId*NB_WRITES+i) + L", '" + fctName+L"')");
    CPPUNIT_ASSERT(nbRowsAffected == 1);
  }
  logInfo(fctName, toWString(NB_WRITES) + L" requests succeeded");

  // check that the writes have been done
  DriverResultSet* drsPtr = statementPtr->executeQuery(
      wstring(L"SELECT * FROM ") + THREAD_TABLE + L" WHERE s='" + fctName + L"'");
  for (int i = 0; i < NB_WRITES; i++)
  {
    CPPUNIT_ASSERT(drsPtr->next());
    logInfo(fctName, drsPtr->getAsString(1) + L"->" + drsPtr->getAsString(2));
  }
  me->instancePtr->connectionPtr->deleteStatement(statementPtr);
  pthread_exit(0);
  return NULL;
}

void TestExecWriteRequest::testThreadedWrite()
{
  const wstring fctName(L"TestExecWriteRequest::testThreadedWrite");
  // create the test table
  createOrReplaceTable(connectionPtr, THREAD_TABLE, THREAD_TABLE_TYPE);
  
  pthread_t threads[NB_THREADS];
  int  retCode[NB_THREADS];
  threadArg args[NB_THREADS];
  /* Create independent threads each of which will execute write function */
  for (int i=0; i<NB_THREADS; i++)
  {
    args[i].instancePtr = this;
    args[i].threadId = i;
    retCode[i] = pthread_create( &(threads[i]), NULL, doThreadedWrites, (void*) &(args[i]));
  }
  // Wait till threads are complete
  for (int i=0; i<NB_THREADS; i++)
  {
    pthread_join(threads[i], NULL);
  }
}
#ifndef CAROB_TEST_USE_MYSQL
void TestExecWriteRequest::testWriteThousandGood()
{
  const wstring fctName(L"TestExecWriteRequest::testWriteThousandGood");
  doNWrites(fctName, 1000);
}
#else
void TestExecWriteRequest::testWriteThousandGood()
{
  const wstring fctName(L"TestExecWriteRequest::testWriteThousandGoodMySQL");

  if (isInfoEnabled())
  {
    logInfo(fctName, L"creating table");
  }
  Statement* statementPtr = connectionPtr->createStatement();
  statementPtr->executeUpdate(L"DROP TABLE IF EXISTS product");
  statementPtr->executeUpdate(L"CREATE TABLE product (id integer, cost integer)");
  for (int j=0; j<50; j++)
    statementPtr->executeUpdate(L"INSERT INTO product VALUES(" + toWString(j) + L","  + toWString(j) + L")");
  connectionPtr->deleteStatement(statementPtr);
  doNWrites(fctName, 1000);
}
#endif
void TestExecWriteRequest::doNWrites(const wstring fctName, int nbWrites)
{
  wstring requestBegin = L"UPDATE product SET cost=";
  wstring requestEnd = L" WHERE id=";
  logInfo(fctName, L"Executing writes");
  Statement* statementPtr = connectionPtr->createStatement();
  for (int i=0; i<nbWrites; i++)
  {
    wstring fullRequest = requestBegin + toWString(i) + requestEnd + toWString(i%50);
    int nbRowsAffected = statementPtr->executeUpdate(fullRequest);
    CPPUNIT_ASSERT(nbRowsAffected == 1);
    logInfo(fctName, L"Request " + fullRequest + L" succeeded");
  }
  connectionPtr->deleteStatement(statementPtr);
  if (isInfoEnabled())
  {
    logInfo(fctName, L"All writes succeeded");
  }
}
  
void TestExecWriteRequest::testLongWriteMySQL()
{
  const wstring fctName(L"TestExecWriteRequest::testLongWriteMySQL");

  Statement* statementPtr = connectionPtr->createStatement();
  statementPtr->executeUpdate(L"DROP TABLE IF EXISTS A");
  statementPtr->executeUpdate(L"CREATE TABLE A (i integer)");
  //createOrReplaceTable(connectionPtr, L"A", L"(dummy integer)");
  if (isInfoEnabled())
  {
    logInfo(fctName, L"Executing 10 s long writes");
  }
  for (int i=0; i<10; i++)
  {
    statementPtr->executeQuery(L"SELECT SLEEP(30)");
  }
  connectionPtr->deleteStatement(statementPtr);
}

CppUnit::Test* TestExecWriteRequest::suite()
{
  CppUnit::TestSuite *suiteOfTests = new CppUnit::TestSuite("20.TestExecWriteRequest");

  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testWriteBadRequest", 
                                 &TestExecWriteRequest::testWriteBadRequest));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testWriteBadTable", 
                                 &TestExecWriteRequest::testWriteBadTable));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testWriteGood", 
                                 &TestExecWriteRequest::testWriteGood));

  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testThreadedWrite", 
                                 &TestExecWriteRequest::testThreadedWrite));
/*  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testWriteThousandGood", 
                                 &TestExecWriteRequest::testWriteThousandGood));
  suiteOfTests->addTest(new CppUnit::TestCaller<TestExecWriteRequest>(
                                 "20.TestExecWriteRequest::testLongWriteMySQL",
                                 &TestExecWriteRequest::testLongWriteMySQL));
*/
  return suiteOfTests;
}
