/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Marc Herbert
 * Contributor(s): 
 */

#include <iostream>
#include <sstream>

#include "CarobException.hpp"
#include <cppunit/Protector.h>
#include <cppunit/Message.h>


static const std::wstring BEPREFIX(L"BackendException: ");
static const std::wstring COPREFIX(L"ControllerException: ");
static const std::wstring CAPREFIX(L"CarobException: ");
static const std::wstring UEPREFIX(L"UnexpectedException: ");
static const std::wstring NEXTPREFIX(L"chained CarobException : ");


static void cerrCarobExceptionChain(const std::wstring& prefix,
    const CarobNS::CarobException& ce)
{
    std::wostringstream woss;

    woss << prefix;
    woss << ce;

    CarobNS::logInfo(L"CppunitCarobProtector::cerrCarobExceptionChain(...)", woss.str());
}


/** This class is a quick & dirty (but working & useful) sketch of a
 * generic CarobException catcher for CPPUnit */

class CarobProtector : public CppUnit::Protector
{

  void reportCppUnitError(const CppUnit::ProtectorContext &context,
                          const std::string& prefix, const CarobNS::CarobException& ce)
  {
    std::wstring sqlState = ce.getSQLState();
    reportError( context, CppUnit::Message(prefix
                                           + CarobNS::toString(ce.description())
                                           + " (SQL State=" + CarobNS::toString(sqlState) + ")"));
  }
  
  bool protect( const CppUnit::Functor &functor,
                const CppUnit::ProtectorContext &context )
  {
    try {
      return functor();
    }
    catch (const CarobNS::BackendException& be)
    {
      // log
      cerrCarobExceptionChain(BEPREFIX, be);
      // CPPunit report
      reportCppUnitError(context, "BackendException:", be);
    }
    catch (const CarobNS::ControllerException& ce)
    {
      // log
      cerrCarobExceptionChain(COPREFIX, ce);
      // CPPunit report
      reportCppUnitError(context, "ControllerException:", ce);
    }
    catch (const CarobNS::CodecException& ce)
    {
      // log
      cerrCarobExceptionChain(COPREFIX, ce);
      // CPPunit report
      reportCppUnitError(context, "CodecException:", ce);
    }
    catch (const CarobNS::UnexpectedException& ue)
    {
      // log
      cerrCarobExceptionChain(UEPREFIX, ue);
      // CPPunit report
      reportCppUnitError(context, "UnexpectedException:", ue);
    }
    // catch all other CarobExceptions
    catch (const CarobNS::CarobException& ce)
    {
      // log
      cerrCarobExceptionChain(CAPREFIX, ce);
      // CPPunit report
      reportCppUnitError(context, "CarobException:", ce);
    }

    return false; // meaning: failed?
  }

};

/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 4
 * indent-tabs-mode: nil
 * End:
 */
