/*
 * Sequoia: Database clustering technology.
 * Copyright (C) 2005-2006 Continuent, Inc.
 * Contact: sequoia@continuent.org
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Initial developer(s): Gilles Rayrat
 * Contributor(s): 
 */

#include "Common.hpp"
#include "ConnectionParameters.hpp"
#include "CarobException.hpp"
#include "Statement.hpp"
#include "DriverResultSet.hpp"

#include "ConnectionSetup.hpp"

#include <vector>

using std::wstring;

using namespace CarobNS;

// statics initialization
#ifdef CAROB_TEST_USE_MYSQL
const std::wstring& ConnectionSetup::DEFAULT_HOST1   = L"node1";
const std::wstring& ConnectionSetup::DEFAULT_HOST2   = L"node2";
const int          ConnectionSetup::DEFAULT_PORT1   = 25322;
const int          ConnectionSetup::DEFAULT_PORT2   = 25322;
const std::wstring& ConnectionSetup::DEFAULT_DB     = L"DB1";
const std::wstring& ConnectionSetup::DEFAULT_USER   = L"user";
const std::wstring& ConnectionSetup::DEFAULT_PASSWD = L"pass";
#else
const std::wstring& ConnectionSetup::DEFAULT_HOST1   = L"localhost";
const std::wstring& ConnectionSetup::DEFAULT_HOST2   = L"localhost";
const int          ConnectionSetup::DEFAULT_PORT1   = 25322;
const int          ConnectionSetup::DEFAULT_PORT2   = 25323;
const std::wstring& ConnectionSetup::DEFAULT_DB     = ConnectionParameters::DEFAULT_DB;
const std::wstring& ConnectionSetup::DEFAULT_USER   = ConnectionParameters::DEFAULT_USER;
const std::wstring& ConnectionSetup::DEFAULT_PASSWD = ConnectionParameters::DEFAULT_PASSWD;
#endif
const CarobNS::ConnectPolicy ConnectionSetup::DEFAULT_POLICY = ConnectionParameters::DEFAULT_POLICY;
const int          ConnectionSetup::DEFAULT_PING_DELAY = ConnectionParameters::DEFAULT_PING_DELAY;
const int          ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT = ConnectionParameters::DEFAULT_CONTROLLER_TIMEOUT;
const bool         ConnectionSetup::DEFAULT_CONNECTION_PERSISTENCY = ConnectionParameters::DEFAULT_CONNECTION_PERSISTENCY;
const bool         ConnectionSetup::DEFAULT_RETRIEVE_SQL_WARNINGS = ConnectionParameters::DEFAULT_RETRIEVE_SQL_WARNINGS;

namespace {
  // try to reuse lastConnection?
  const bool   reuseConnection  = true;
  Connection*  lastConnection   = NULL;
}


void ConnectionSetup::setUp()
{
  static const wchar_t fctName[] = L"ConnectionSetup::setUp()";

  if (reuseConnection)
  {
    if (lastConnection != NULL
    && ! lastConnection->isClosed())
    {
      logInfo(fctName, L"re-using lastConnection");
      connectionPtr = lastConnection;
      return; // reuse success
    }
     // reuse failed: fallback to reconnect below
    else if (lastConnection == NULL)
      logWarn(fctName, L"failed to re-use lastConnection, was NULL");
    else
      logError(fctName, L"failed to re-use lastConnection, was closed");
  }

  std::vector<ControllerInfo> ctrls;
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST1, DEFAULT_PORT1));
  ctrls.push_back(ControllerInfo(ConnectionSetup::DEFAULT_HOST2, DEFAULT_PORT2));
  ConnectionParameters connectionPrms(ctrls,
                                      ConnectionSetup::DEFAULT_DB,
                                      ConnectionSetup::DEFAULT_USER,
                                      ConnectionSetup::DEFAULT_PASSWD,
                                      ConnectionSetup::DEFAULT_POLICY,
                                      ConnectionSetup::DEFAULT_PING_DELAY,
                                      ConnectionSetup::DEFAULT_CONTROLLER_TIMEOUT,
                                      ConnectionSetup::DEFAULT_CONNECTION_PERSISTENCY,
                                      ConnectionSetup::DEFAULT_RETRIEVE_SQL_WARNINGS);
  try
  {
    connectionPtr = new Connection(connectionPrms);
  }
  catch (CarobException e)
  {
    logError(L"ConnectionSetup::setUp()", L"Connection failed: " + e.description()); 
    connectionPtr = NULL;
    CPPUNIT_ASSERT_MESSAGE("Could not connect to default host", false);
  }

  lastConnection = connectionPtr;

}

void ConnectionSetup::testConnection()
{
  Statement* teststmt = connectionPtr->createStatement();

  // FIXME: product table is created in CarobTestLauncher
  DriverResultSet* drsPtr =
    teststmt->executeQuery(L"SELECT * FROM product WHERE ID<2");

  drsPtr->next();

  connectionPtr->deleteStatement(teststmt);
}

void ConnectionSetup::tearDown()
{
  testConnection();

  if (reuseConnection)
    return;

  delete connectionPtr;
  lastConnection = NULL;
}

void dropTableIfExists(Connection *conn_p, const std::wstring& tableName)
{
#ifdef CAROB_TEST_USE_MYSQL
  std::wstring dropcommand(L"drop table if exists ");
#else
  std::wstring dropcommand(L"drop table ");
#endif    
  dropcommand += tableName;

  Statement* dropstmt = conn_p->createStatement();

  try
  {
    int updatecount = dropstmt->executeUpdate(dropcommand);
    CPPUNIT_ASSERT(0 == updatecount);
  }
  catch (const BackendException& be)
  {
    // rethrow if different from UNDEFINED TABLE
    // TODO: ignore case here. Problem: "case" concept is locale sensitive?
    if (0 != be.getSQLState().compare(L"42P01") // postgres
        && 0 != be.getSQLState().compare(L"S0002")) // hypersonic
      throw be; // and leak. Who cares?
    // else ignore
  }

  conn_p->deleteStatement(dropstmt);
}

void createOrReplaceTable(Connection *conn_p, const std::wstring& tableName, const std::wstring& tableType)
{
  dropTableIfExists(conn_p, tableName);

  std::wstring createcommand(L"CREATE TABLE ");
  createcommand += tableName + L" ";
  createcommand += tableType;
  
  Statement* createstmt = conn_p->createStatement();

  int updatecount = createstmt->executeUpdate(createcommand);
  CPPUNIT_ASSERT(0 == updatecount);

  conn_p->deleteStatement(createstmt);
}


/*
 * Local Variables:
 * c-file-style: "bsd"
 * c-basic-offset: 2
 * indent-tabs-mode: nil
 * End:
 */
